###############################################################################
#                                                                             #
# OpenMOIV .PRO file (Makefile/DSP Generator)                                 #
#                                                                             #
# This is the source file to generate the Makefile for your target platform.  #
# It uses the qmake (AKA tmake) program from Trolltech (you can found it      #
# inside the Qt/X11, Qt/Win or Qt/Mac packages).                              #
#                                                                             #
#=============================================================================#
# Instructions:                                                               #
#                                                                             #
#  $ qmake moiv2.pro   (add here all your modifiers!)                         #
#  $ make              (or nmake)                                             #
#                                                                             #
#=============================================================================#
# Modifiers:
# ............................................................................#
# DEFINES += __no_libFL                                                       #
# win32:DEFINES += __sys_fonts __WGL                                          #
# unix:DEFINES += __sys_fonts __GLX                                           #
# ............................................................................#
#                                                                             #
# Notes: (if none of the above is defined, the library will link with libFL)  #
# - if you define "__no_libFL" :                                              #
#   Don't use the libFL for fonts, instead it uses the minilibFL wrapper      #
#   This support only one fixed-font hardcoded in the source                  #
# - if you define "__sys_fonts" :                                             #
#   Use system fonts, don't use neither libFL nor minilibFL wrapper           #
#   If you use system fonts, you must define a symbol depending OS (values)   #
#                                                                             #
# ..........................................................................  #
# DEFINES += __coin							      #
# ............................................................................#
#                                                                             #
# Notes: Enable compile support with COIN instead SGI/TGS-OIV                 #
#                                                                             #
#=============================================================================#
# Examples:                                                                   #
#=============================================================================#
# ..........................................................................  #
# Platform: Linux (static library)                                            #
# Inventor version: SGI-OSS-OIV                                               #
# System fonts: One harcoded font                                             #
#                                                                             #
# DEFINES += __no_libFL                                                       #
# CFLAGS += -mcpu=i686                                                        #
# ..........................................................................  #
# Platform: Linux (dynamic library)                                           #
# Inventor version: Coin                                                      #
# System fonts: GLX							      #
#                                                                             #
# DEFINES += __coin                                                           #
# DEFINES += __sys_fonts __GLX                                                #
# CFLAGS += -mcpu=i686                                                        #
# LIBS += -shared                                                             #
# ..........................................................................  #
# Platform: Windows (static library)                                          #
# Inventor version: Coin                                                      #
# System fonts: One hardcoded font                                            #
#                                                                             #
# DEFINES += __coin                                                           #
# DEFINES += __no_libFL                                                       #
# DEFINES += _CHEMKIT_INTERNAL                                                #
# ..........................................................................  #
# Platform: Windows (dynamic library)                                         #
# Inventor version: Coin                                                      #
# System fonts: WGL							      #
#                                                                             #
# DEFINES += __coin                                                           #
# DEFINES += __sys_fonts __WGL                                                #
# DEFINES += _CHEMKIT_INTERNAL _MAKE_CHEMKIT_DLL                              #
# ..........................................................................  #
# Platform: IRIX (dynamic library)                                            #
# Inventor version: SGI-OSS-OIV                                               #
# System fonts: Fl                                                            #
#                                                                             #
# DEFINES += __sgi                                                            #
# CFLAGS += -mips3                                                            #
# LIBS += -shared                                                             #
# ..........................................................................  #
# Platform: MacOS X (dynamic library)                                         #
# Inventor version: Coin                                                      #
# System fonts: One hardcoded font                                            #
#                                                                             #
# Notes about MacOX X configurations:                                         #
# It's mandatory to use Coin and one harcoded font.                           #
# System fonts support using AGL will be added in future releases.            #
# ..........................................................................  #
#                                                                             #
###############################################################################


#############################
### Edit only if you know ###
#############################

# use a replacement of rw bitvec class
DEFINES += __no_rw

# Don't interrogate to SoXtViewer about the stereo state
# Stereo may affect the lasso rendering
# This removes all dependences to SoXt!
DEFINES += __nogl

# add openmoiv include path
INCLUDEPATH += ./include 

# win32
win32:DEFINES	+= _WINDOWS WIN32 IV2_6
win32:INCLUDEPATH += $(OIV_DIR)/include
win32:LIBS += opengl32.lib glu32.lib gdi32.lib oleaut32.lib

# unix 
unix:!mac:INCLUDEPATH += $(OIV_DIR)/include
unix:!mac:LIBS	+= -lGL -lm

# mac (use coin and hardcoded font)
mac:DEFINES += __no_libFL __no_rw __nogl __coin
mac:INCLUDEPATH += /usr/include/sys
mac:INCLUDEPATH += /System/Library/Frameworks/OpenGL.framework/Versions/A/Headers/
mac:INCLUDEPATH += /usr/X11R6/include
mac:QMAKE_LFLAGS += -framework Inventor
mac:QMAKE_LFLAGS += -framework OpenGL

# Select Inventor or Coin for UNIX and win32
contains( DEFINES, __coin ) {
	unix:!mac:LIBS += -lCoin
	win32:DEFINES += COIN_DLL
	win32:LIBS += $(OIV_DIR)/lib/coin2.lib
} else {
	unix:!mac:LIBS += -lInventor
	win32:LIBS += $(OIV_DIR)/lib/inventor.lib
}

# If you like to use libFL also include it for UNIX
!contains( DEFINES, __no_libFL ) {
  !contains( DEFINES, __sys_fonts ) {
	unix:!mac:LIBS	+= -lFL
  }
}

############################
### Don't edit from here ###
############################

# Include the source code of our BitVec class
contains( DEFINES, __no_rw ) {
	INCLUDEPATH += ./stuff
}

# Debug/Release values
debug {
	DEFINES	+= _DEBUG
	CONFIG	-= release
	CONFIG	+= warn_on
} else {
	DEFINES	+= NDEBUG
	CONFIG	-= warn_on
	CONFIG	+= release warn_off
}

# Don't depends on Qt --> don't include any Qt stuff
CONFIG -= qt

# Data of the output library
TEMPLATE = lib
TARGET = ChemKit2
VERSION = 1.0.0
DESTDIR = ./lib

# Additional qmake info
MOC_DIR		= .moc
OBJECTS_DIR	= .obj

	
SOURCES  += \
      ./src/SbAtomSpec.cpp \
      ./src/SFAtomSpec.cpp\
      ./src/MFAtomSpec.cpp\
      ./src/SFVec2AtomSpec.cpp\
      ./src/MFVec2AtomSpec.cpp\
      ./src/SFVec3AtomSpec.cpp\
      ./src/MFVec3AtomSpec.cpp\
      ./src/SFVec4AtomSpec.cpp\
      ./src/MFVec4AtomSpec.cpp\
      ./src/SbVecI.cpp\
      ./src/SFDouble.cpp\
      ./src/MFDouble.cpp\
      ./src/SFInt8.cpp\
      ./src/MFInt8.cpp\
      ./src/SFVec2i.cpp\
      ./src/MFVec2i.cpp\
      ./src/SFVec3i.cpp\
      ./src/MFVec3i.cpp\
      ./src/SFVec4i.cpp\
      ./src/MFVec4i.cpp\
      ./src/ChemIntList.cpp\
      ./src/ChemAromaticRing.cpp\
      ./src/ChemAssociatedData.cpp\
      ./src/ChemAssociatedDataElement.cpp\
      ./src/ChemAtomBBox.cpp\
      ./src/ChemBaseData.cpp\
      ./src/ChemBaseDataElement.cpp\
      ./src/ChemBitmapFontCache.cpp\
      ./src/ChemBBoxAction.cpp\
      ./src/ChemBondBBox.cpp\
      ./src/ChemColor.cpp\
      ./src/ChemColorElement.cpp\
      ./src/ChemCondenser.cpp\
      ./src/ChemConnollyDot.cpp\
      ./src/ChemContour.cpp\
      ./src/ChemContour2.cpp\
      ./src/ChemData.cpp\
      ./src/ChemDetail.cpp\
      ./src/ChemDisplay.cpp\
      ./src/ChemDisplayCylinders.cpp\
      ./src/ChemDisplaySingleAtoms.cpp\
      ./src/ChemDisplaySpheres.cpp\
      ./src/ChemDisplayText.cpp\
      ./src/ChemDisplayWireframe.cpp\
      ./src/ChemDisplayKit.cpp\
      ./src/ChemDisplayParam.cpp\
      ./src/ChemDisplayParamElement.cpp\
      ./src/ChemDisplayPath.cpp\
      ./src/ChemDisplayPathList.cpp\
      ./src/ChemDisplaySelectionElement.cpp\
      ./src/ChemHasher.cpp\
      ./src/ChemInit.cpp\
      ./src/ChemIso.cpp\
      ./src/ChemKit.cpp\
      ./src/ChemLabel.cpp\
      ./src/ChemLabelBBox.cpp\
      ./src/ChemLabelPath.cpp\
      ./src/ChemLabelPathList.cpp\
      ./src/ChemLabelSelectionElement.cpp\
      ./src/ChemLattice3.cpp\
      ./src/ChemLattice3Element.cpp\
      ./src/ChemMonitor.cpp\
      ./src/ChemMonitorBBox.cpp\
      ./src/ChemMonitorDetail.cpp\
      ./src/ChemMonitorPath.cpp\
      ./src/ChemMonitorPathList.cpp\
      ./src/ChemMonitorSelectionElement.cpp\
      ./src/ChemOverrideElement.cpp\
      ./src/ChemPath.cpp\
      ./src/ChemRadii.cpp\
      ./src/ChemRadiiElement.cpp\
      ./src/ChemSelection.cpp\
      ./src/ChemSelectionElement.cpp\
      ./src/ChemSpewCylinders.cpp\
      ./src/ChemSpewWireframe.cpp\
      ./src/ChemSSSR.cpp\
      ./src/ChemStickBondBBox.cpp\
      ./src/ChemUnitCylinder.cpp\
      ./src/ChemUnitSphere.cpp\
      ./src/ChemWireframeBondBBox.cpp\
      ./src/ChemWireframeDetail.cpp\
      ./src/ConnollyDotFxns.cpp\
      ./src/ContourLib.cpp\
      ./src/IsoLib.cpp\
      ./src/bitvec.cpp\
      ./src/minilibFL.cpp\
      ./src/ChemLOD.cpp\
      ./src/ChemDisplaySpheresLOD.cpp\
      ./src/ChemDisplayCylindersLOD.cpp\
      ./src/ChemDisplayResidues.cpp\
      ./src/SbResidue.cpp\
      ./src/SbChain.cpp\
      ./src/ChemSAS.cpp\
      ./src/ChemBitmapFontCacheSys.cpp\
      ./src/sysfonts.cpp\
      ./src/ChemOctreeNode.cpp\
      ./src/ChemResidueStuff.cpp\
      ./src/ChemFileImporter.cpp\
      ./src/ChemPDBImporter.cpp\
      ./src/ChemMOLImporter.cpp
