/**************************************************************************\
 *
 * OpenMOIV - C++ library for molecular visualization using Inventor.
 * Copyright (C) 2001-2003 Universitat Pompeu Fabra - Barcelona (Spain)
 * 
 * Developers: Interactive Technology Group (GTI)
 * Team: Josep Blat, Eduard Gonzalez, Sergi Gonzalez,
 *       Daniel Soto, Alejandro Ramirez, Oscar Civit.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details (see the file 
 * LICENSE.LGPL at the root directory).
 *
 * REMARK: This library is a derived product.
 *         You need also to accept all other applicable licenses.
 * 
 * Homepage: http://www.tecn.upf.es/openMOIV/
 * Contact:  openmoiv@upf.es
 *
\**************************************************************************/

#ifdef __no_rw

#include <rw/bitvec.h>
#include <string.h>

#include <assert.h>

RWBitVec::RWBitVec()
{
	data_ = NULL;
	length_ = 0;
}

RWBitVec::RWBitVec(size_t N, RWBoolean _value)
{
	int i;
	RWByte value = _value ? 1:0;

	length_ = N;
	data_ = new RWByte[length_];
	assert(data_!=NULL);

	for (i=0; i<length_; i++)
		data_[i] = value;
}

RWBitVec::RWBitVec(const RWBitVec& bv)
{
	length_ = bv.length_;
	data_ = new RWByte[length_];

	int i;
	for (i=0; i<length_; i++)
		data_[i] = bv[i];
}


RWBitVec::~RWBitVec()
{
	if (data_ != NULL)
		delete[] data_;
	data_ = NULL;

}

RWBitVec&             RWBitVec::operator=(const RWBitVec& bv)
{
	resize(bv.length_);
	memcpy(data_, bv.data_, length_);

	return *this;
}

RWBitVec&             RWBitVec::operator^=(const RWBitVec& bv)
{
	int i;
	for (i=0; i<length_; i++)
		data_[i] = data_[i] ^ bv.data_[i];

	return *this;
}

RWBitVec&             RWBitVec::operator|=(const RWBitVec& bv)
{
	int i;

	for (i=0; i<length_ ; i++)
		data_[i] = data_[i] | bv.data_[i];

	return *this;
}

RWBitRef              RWBitVec::operator[](size_t i)
{
	assert(i>=0 && i<length_);
	return RWBitRef(&data_[i]);
}

RWBitRef              RWBitVec::operator()(size_t i)
{
	assert(i>=0 && i<length_);
	return RWBitRef(&data_[i]);
}

RWBoolean        RWBitVec::operator[](size_t i) const
{
	assert(i>=0 && i<length_);
	return data_[i];
}

RWBoolean        RWBitVec::operator()(size_t i) const
{
	assert(i>=0 && i<length_);
	return data_[i];
}

RWBoolean             RWBitVec::operator!=(const RWBitVec& bv) const
{
	return !isEqual(bv);
}

void                  RWBitVec::clearBit(size_t i)
{
	assert(i>=0 && i<length_);
	data_[i] = 0;
}

size_t                RWBitVec::firstFalse() const
{
	int i;

	for (i=0; i<length_; i++)
		if (data_[i]==0)
			break;

	return i;
}

size_t                RWBitVec::firstTrue() const
{
	int i;
	for (i=0; i<length_; i++)
		if (data_[i]==1)
			break;

	return i;
}

unsigned              RWBitVec::hash() const
{
	return 0;
}


RWBoolean             RWBitVec::isEqual(const RWBitVec& bv) const
{
	if (length_!=bv.length_)
		return false;

	int i;

	for (i=0; i<length_; i++)
		if (data_[i] != bv.data_[i])
			return false;

	return true;
}

size_t                RWBitVec::length() const
{
	return length_;
}

void                  RWBitVec::resize(size_t newSize)
{
	if (newSize==length_)
		return;

	RWByte *d = new RWByte[newSize];
	memcpy(d, data_, length_);

	for (int i=length_; i<newSize; i++)
		d[i] = 0;

	delete[] data_;

	data_ = d;
	length_ = newSize;
}

void                  RWBitVec::setBit(size_t i)
{
	assert(i>=0 && i<length_);
	data_[i] = 1;
}

RWBoolean             RWBitVec::testBit(size_t i) const
{
	assert(i>=0 && i<length_);
	return data_[i] != 0;
}

size_t         sum(const RWBitVec& V)
{
	int i, j=0;
	
	for (i=0; i<V.length_; i++)
		j += V.data_[i];

	return j;
}

#endif	// __no_rw
