/**************************************************************************\
 *
 * OpenMOIV - C++ library for molecular visualization using Inventor.
 * Copyright (C) 2001-2003 Universitat Pompeu Fabra - Barcelona (Spain)
 * 
 * Developers: Interactive Technology Group (GTI)
 * Team: Josep Blat, Eduard Gonzalez, Sergi Gonzalez,
 *       Daniel Soto, Alejandro Ramirez, Oscar Civit.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details (see the file 
 * LICENSE.LGPL at the root directory).
 *
 * REMARK: This library is a derived product.
 *         You need also to accept all other applicable licenses.
 * 
 * Homepage: http://www.tecn.upf.es/gti/openmoiv/
 * Contact:  openmoiv@upf.es
 *
\**************************************************************************/

#ifndef __RWBITVEC_H__
#define __RWBITVEC_H__

// simple implementation for a vector of bits
#include <stdio.h> // size_t

typedef unsigned char  RWByte;
typedef int RWBoolean;

class RWBitRef {
public:

  RWBitRef(RWByte* p)
    { pt_ = p; }

  RWBitRef(const RWBitRef& br)
    { pt_=br.pt_; }

  operator      RWBoolean() const
    { return *pt_!= 0; }

  RWBoolean     operator=(RWBoolean i)
    { if(i) *pt_ = 1;
    else    *pt_ = 0;
    return i; }

  void          operator&=(RWBoolean i) {if (!i) *pt_ &= ~1;}
  void          operator|=(RWBoolean i) {if  (i) *pt_ |=  1;}
  void          operator^=(RWBoolean i) {if  (i) *pt_ ^=  1;}
private:
  RWByte*       pt_;

  friend class RWBitVec;
};


class RWBitVec
{
	RWByte *data_;
	int length_;
public:

  RWBitVec();
  RWBitVec(size_t N, RWBoolean value);
  RWBitVec(const RWBitVec&);
  ~RWBitVec();

  // Assignment operators
  RWBitVec&             operator=(const RWBitVec&);
  RWBitVec&             operator^=(const RWBitVec& bv);
  RWBitVec&             operator|=(const RWBitVec& bv);

  // Indexing operators
  RWBitRef              operator[](size_t i);   // Can be used as lvalue
  RWBitRef              operator()(size_t i);   //       ditto

  RWBoolean             operator[](size_t i) const; // Can't be used as lvalue
  RWBoolean             operator()(size_t i) const; //       ditto

  // Other operators ---
  RWBoolean             operator!=(const RWBitVec& bv) const;

  // Member functions
  void                  clearBit(size_t i);             // Clear bit i
  size_t                firstFalse() const;             // Find first OFF bit
  size_t                firstTrue() const;              // Find first ON  bit
  unsigned              hash() const;
  RWBoolean             isEqual(const RWBitVec& bv) const;
  size_t                length() const;
  void                  resize(size_t newSize);
  void                  setBit(size_t i);                       // Set bit i
  RWBoolean             testBit(size_t i) const;                // Return value of bit i

  // Other friend functions:
  friend size_t			sum(const RWBitVec&);           // Total # of bits set
};

#endif /* __RWBITVEC_H__ */

